@@ -21,13 +21,10 @@
 #include <string.h>
 #include "nautilus-icon-info.h"
 #include "nautilus-default-file-icon.h"
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtkiconfactory.h>
+#include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 
-#define NAUTILUS_EMBLEM_NAME_PREFIX "emblem-"
-
 struct _NautilusIconInfo
 {
 	GObject parent;
@@ -353,10 +350,7 @@ nautilus_icon_info_lookup (GIcon *icon,
 					       size,
 					       NULL, NULL, NULL);
 		if (stream) {
-			pixbuf = eel_gdk_pixbuf_load_from_stream (stream);
-
-			/* TODO: resize icon? */
-
+			pixbuf = eel_gdk_pixbuf_load_from_stream_at_size (stream, size);
 			g_object_unref (stream);
 		}
 
@@ -494,7 +488,7 @@ nautilus_icon_info_get_pixbuf_nodefault_at_size (NautilusIconInfo  *icon,
 	scale = (double)forced_size / s;
 	scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
 						 w * scale, h * scale,
-						 GDK_INTERP_HYPER);
+						 GDK_INTERP_BILINEAR);
 	g_object_unref (pixbuf);
 	return scaled_pixbuf;
 }
@@ -520,7 +514,7 @@ nautilus_icon_info_get_pixbuf_at_size (NautilusIconInfo  *icon,
 	scale = (double)forced_size / s;
 	scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
 						 w * scale, h * scale,
-						 GDK_INTERP_HYPER);
+						 GDK_INTERP_BILINEAR);
 	g_object_unref (pixbuf);
 	return scaled_pixbuf;
 }
@@ -649,16 +643,6 @@ nautilus_get_icon_size_for_stock_size (GtkIconSize size)
 }
 
 
-char *
-nautilus_icon_get_emblem_icon_by_name (const char *emblem_name)
-{
-	char *name_with_prefix;
-
-	name_with_prefix = g_strconcat (NAUTILUS_EMBLEM_NAME_PREFIX, emblem_name, NULL);
-
-	return name_with_prefix;
-}
-
 guint
 nautilus_icon_get_emblem_size_for_icon_size (guint size)
 {
@@ -668,8 +652,10 @@ nautilus_icon_get_emblem_size_for_icon_size (guint size)
 		return 32;
 	if (size >= 48)
 		return 24;
-	if (size >= 32)
+	if (size >= 24)
 		return 16;
+	if (size >= 16)
+		return 12;
 	
 	return 0; /* no emblems for smaller sizes */
 }
