@@ -956,20 +956,32 @@ activation_start_timed_cancel (ActivateParameters *parameters)
 		 parameters->parent_window);
 }
 
+static void
+pause_activation_timed_cancel (ActivateParameters *parameters)
+{
+	if (parameters->timed_wait_active) {
+		eel_timed_wait_stop (cancel_activate_callback, parameters);
+		parameters->timed_wait_active = FALSE;
+	}
+}
+
+static void
+unpause_activation_timed_cancel (ActivateParameters *parameters)
+{
+	if (!parameters->timed_wait_active) {
+		activation_start_timed_cancel (parameters);
+	}
+}
+
 static void
 activate_mount_op_active (EelMountOperation *operation,
 			  gboolean is_active,
 			  ActivateParameters *parameters)
 {
 	if (is_active) {
-		if (parameters->timed_wait_active) {
-			eel_timed_wait_stop (cancel_activate_callback, parameters);
-			parameters->timed_wait_active = FALSE;
-		}
+		pause_activation_timed_cancel (parameters);
 	} else {
-		if (!parameters->timed_wait_active) {
-			activation_start_timed_cancel (parameters);
-		}
+		unpause_activation_timed_cancel (parameters);
 	}
 }
 
@@ -1048,7 +1060,9 @@ activate_files (ActivateParameters *parameters)
 			/* Special case for executable text files, since it might be
 			 * dangerous & unexpected to launch these.
 			 */
+			pause_activation_timed_cancel (parameters);
 			action = get_executable_text_file_action (parameters->parent_window, file);
+			unpause_activation_timed_cancel (parameters);
 		}
 
 		switch (action) {
