@@ -234,8 +234,7 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 				   FMIconView *icon_view)
 {
 	char *position_string, *scale_string;
-	gboolean position_good, scale_good;
-	char *locale;
+	gboolean position_good;
 	char c;
 
 	g_assert (NAUTILUS_IS_ICON_CONTAINER (container));
@@ -247,15 +246,6 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 		return FALSE;
 	}
 
-	/* Doing parsing in the "C" locale instead of the one set
-	 * by the user ensures that data in the metafile is not in
-	 * a locale-specific format. It's only necessary for floating
-	 * point values since there aren't locale-specific formats for
-	 * integers in C stdio.
-	 */
-	locale = g_strdup (setlocale (LC_NUMERIC, NULL));	
-	setlocale (LC_NUMERIC, "C");
-
 	/* Get the current position of this icon from the metadata. */
 	position_string = nautilus_file_get_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_POSITION, "");
@@ -269,21 +259,17 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 	/* Get the scale of the icon from the metadata. */
 	scale_string = nautilus_file_get_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_SCALE, "1");
-	scale_good = sscanf
-		(scale_string, " %lf",
-		 &position->scale) == 1;
-	if (!scale_good) {
-		position->scale = 1.0;
-	}
+	position->scale = g_ascii_strtod (scale_string, NULL);
+	if (errno != 0) {
+ 		position->scale = 1.0;
+ 	}
 
 	g_free (scale_string);
-
-	setlocale (LC_NUMERIC, locale);
-	g_free (locale);
 	
 	return position_good;
 }
 
+
 static gboolean
 set_sort_criterion (FMIconView *icon_view, const SortCriterion *sort)
 {
@@ -510,28 +496,6 @@ fm_icon_view_remove_file (FMDirectoryView *view, NautilusFile *file, NautilusDir
 	}
 }
 
-static gboolean
-file_has_lazy_position (FMDirectoryView *view,
-			NautilusFile *file)
-{
-	gboolean lazy_position;
-
-	/* For volumes (i.e. cdrom icon) we use lazy positioning so that when
-	 * an old cdrom gets re-mounted in a place that now has another
-	 * icon we don't overlap that one. We don't do this in general though,
-	 * as it can cause icons moving around.
-	 */
-	lazy_position = nautilus_file_can_unmount (file);
-	if (lazy_position && fm_directory_view_get_loading (view)) {
-		/* if volumes are loaded during directory load, don't mark them
-		 * as lazy. This is wrong for files that were mounted during user
-		 * log-off, but it is right for files that were mounted during login. */
-		lazy_position = FALSE;
-	}
-
-	return lazy_position;
-}
-
 static void
 fm_icon_view_add_file (FMDirectoryView *view, NautilusFile *file, NautilusDirectory *directory)
 {
@@ -554,8 +518,7 @@ fm_icon_view_add_file (FMDirectoryView *view, NautilusFile *file, NautilusDirect
 	}
 
 	if (nautilus_icon_container_add (icon_container,
-					 NAUTILUS_ICON_CONTAINER_ICON_DATA (file),
-					 file_has_lazy_position (view, file)))  {
+					 NAUTILUS_ICON_CONTAINER_ICON_DATA (file))) {
 		nautilus_file_ref (file);
 	}
 }
@@ -1143,6 +1106,8 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	file = fm_directory_view_get_directory_as_file (view);
 	icon_container = GTK_WIDGET (get_icon_container (icon_view));
 
+	nautilus_icon_container_begin_loading (NAUTILUS_ICON_CONTAINER (icon_container));
+
 	nautilus_icon_container_set_allow_moves (NAUTILUS_ICON_CONTAINER (icon_container),
 						 fm_directory_view_get_allow_moves (view));
 
@@ -1221,11 +1186,16 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 }
 
 static void
-fm_icon_view_end_loading (FMDirectoryView *view)
+fm_icon_view_end_loading (FMDirectoryView *view,
+			  gboolean all_files_seen)
 {
 	FMIconView *icon_view;
+	GtkWidget *icon_container;
 
 	icon_view = FM_ICON_VIEW (view);
+
+	icon_container = GTK_WIDGET (get_icon_container (icon_view));
+	nautilus_icon_container_end_loading (NAUTILUS_ICON_CONTAINER (icon_container), all_files_seen);
 }
 
 static NautilusZoomLevel
@@ -2195,8 +2165,7 @@ fm_icon_view_screen_changed (GtkWidget *widget,
 				fm_icon_view_remove_file (view, file, directory);
 			} else {
 				if (nautilus_icon_container_add (icon_container,
-								 NAUTILUS_ICON_CONTAINER_ICON_DATA (file),
-								 file_has_lazy_position (view, file))) {
+								 NAUTILUS_ICON_CONTAINER_ICON_DATA (file))) {
 					nautilus_file_ref (file);
 				}
 			}
@@ -2267,22 +2236,12 @@ icon_position_changed_callback (NautilusIconContainer *container,
 				FMIconView *icon_view)
 {
 	char *position_string;
-	char *scale_string;
-	char *locale;
+	char scale_string[G_ASCII_DTOSTR_BUF_SIZE];
 
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 	g_assert (container == get_icon_container (icon_view));
 	g_assert (NAUTILUS_IS_FILE (file));
 
-	/* Doing formatting in the "C" locale instead of the one set
-	 * by the user ensures that data in the metafile is not in
-	 * a locale-specific format. It's only necessary for floating
-	 * point values since there aren't locale-specific formats for
-	 * integers in C stdio.
-	 */
-	locale = g_strdup (setlocale (LC_NUMERIC, NULL));	
-	setlocale (LC_NUMERIC, "C");
-
 	/* Schedule updating menus for the next idle. Doing it directly here
 	 * noticeably slows down icon stretching.  The other work here to
 	 * store the icon position and scale does not seem to noticeably
@@ -2307,18 +2266,11 @@ icon_position_changed_callback (NautilusIconContainer *container,
 		g_free (position_string);
 	}
 
-	/* FIXME bugzilla.gnome.org 40662: 
-	 * %.2f is not a good format for the scale factor. We'd like it to
-	 * say "2" or "2x" instead of "2.00".
-	 */
-	scale_string = g_strdup_printf ("%.2f", position->scale);
+
+	g_ascii_dtostr (scale_string, sizeof (scale_string), position->scale);
 	nautilus_file_set_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_SCALE,
-		 "1.00", scale_string);
-	g_free (scale_string);
-
-	setlocale (LC_NUMERIC, locale);
-	g_free (locale);
+		 "1.0", scale_string);
 }
 
 /* Attempt to change the filename to the new text.  Notify user if operation fails. */
@@ -2605,6 +2557,62 @@ fm_icon_view_update_click_mode (FMIconView *icon_view)
 						       click_mode == NAUTILUS_CLICK_POLICY_SINGLE);
 }
 
+static gboolean
+get_stored_layout_timestamp (NautilusIconContainer *container,
+			     NautilusIconData *icon_data,
+			     time_t *timestamp,
+			     FMIconView *view)
+{
+	NautilusFile *file;
+	NautilusDirectory *directory;
+
+	if (icon_data == NULL) {
+		directory = fm_directory_view_get_model (FM_DIRECTORY_VIEW (view));
+		if (directory == NULL) {
+			return FALSE;
+		}
+
+		file = nautilus_directory_get_corresponding_file (directory);
+		*timestamp = nautilus_file_get_time_metadata (file,
+							      NAUTILUS_METADATA_KEY_ICON_VIEW_LAYOUT_TIMESTAMP);
+		nautilus_file_unref (file);
+	} else {
+		*timestamp = nautilus_file_get_time_metadata (NAUTILUS_FILE (icon_data),
+							      NAUTILUS_METADATA_KEY_ICON_POSITION_TIMESTAMP);
+	}
+
+	return TRUE;
+}
+
+static gboolean
+store_layout_timestamp (NautilusIconContainer *container,
+			NautilusIconData *icon_data,
+			const time_t *timestamp,
+			FMIconView *view)
+{
+	NautilusFile *file;
+	NautilusDirectory *directory;
+
+	if (icon_data == NULL) {
+		directory = fm_directory_view_get_model (FM_DIRECTORY_VIEW (view));
+		if (directory == NULL) {
+			return FALSE;
+		}
+
+		file = nautilus_directory_get_corresponding_file (directory);
+		nautilus_file_set_time_metadata (file,
+						 NAUTILUS_METADATA_KEY_ICON_VIEW_LAYOUT_TIMESTAMP,
+						 (time_t) *timestamp);
+		nautilus_file_unref (file);
+	} else {
+		nautilus_file_set_time_metadata (NAUTILUS_FILE (icon_data),
+						 NAUTILUS_METADATA_KEY_ICON_POSITION_TIMESTAMP,
+						 (time_t) *timestamp);
+	}
+
+	return TRUE;
+}
+
 static NautilusIconContainer *
 create_icon_container (FMIconView *icon_view)
 {
@@ -2658,6 +2666,11 @@ create_icon_container (FMIconView *icon_view)
 				 G_CALLBACK (fm_directory_view_update_menus), icon_view,
 				 G_CONNECT_SWAPPED);
 
+	g_signal_connect_object (icon_container, "get_stored_layout_timestamp",
+				 G_CALLBACK (get_stored_layout_timestamp), icon_view, 0);
+	g_signal_connect_object (icon_container, "store_layout_timestamp",
+				 G_CALLBACK (store_layout_timestamp), icon_view, 0);
+
 	gtk_container_add (GTK_CONTAINER (icon_view),
 			   GTK_WIDGET (icon_container));
 
@@ -2749,6 +2762,8 @@ fm_icon_view_set_property (GObject         *object,
 			icon_view->details->compact = g_value_get_boolean (value);
 			if (icon_view->details->compact) {
 				nautilus_icon_container_set_layout_mode (get_icon_container (icon_view),
+									 gtk_widget_get_direction (GTK_WIDGET(icon_view)) == GTK_TEXT_DIR_RTL ?
+									 NAUTILUS_ICON_LAYOUT_T_B_R_L :
 									 NAUTILUS_ICON_LAYOUT_T_B_L_R);
 				nautilus_icon_container_set_forced_icon_size (get_icon_container (icon_view),
 									      NAUTILUS_ICON_SIZE_SMALLEST);
