@@ -29,6 +29,7 @@
 #define NAUTILUS_WINDOW_PRIVATE_H
 
 #include "nautilus-window.h"
+#include "nautilus-window-slot.h"
 #include "nautilus-spatial-window.h"
 #include "nautilus-navigation-window.h"
 
@@ -37,23 +38,12 @@
 #include <bonobo/bonobo-ui-toolbar-button-item.h>
 #include <libnautilus-private/nautilus-directory.h>
 
-typedef enum {
-        NAUTILUS_LOCATION_CHANGE_STANDARD,
-        NAUTILUS_LOCATION_CHANGE_BACK,
-        NAUTILUS_LOCATION_CHANGE_FORWARD,
-        NAUTILUS_LOCATION_CHANGE_RELOAD,
-        NAUTILUS_LOCATION_CHANGE_REDIRECT,
-        NAUTILUS_LOCATION_CHANGE_FALLBACK
-} NautilusLocationChangeType;
-
 /* FIXME bugzilla.gnome.org 42575: Migrate more fields into here. */
 struct NautilusWindowDetails
 {
         GtkWidget *table;
         GtkWidget *statusbar;
         GtkWidget *menubar;
-
-        GtkWidget *extra_location_widgets;
         
         GtkUIManager *ui_manager;
         GtkActionGroup *main_action_group; /* owned by ui_manager */
@@ -66,43 +56,26 @@ struct NautilusWindowDetails
         GtkActionGroup *bookmarks_action_group;
         guint refresh_bookmarks_menu_idle_id;
         guint bookmarks_merge_id;
-        
-        /* Current location. */
-        GFile *location;
-	char *title;
-	NautilusFile *viewed_file;
-        gboolean viewed_file_seen;
-	gboolean viewed_file_in_trash;
-	gboolean allow_stop;
 
-        /* New location. */
-        NautilusLocationChangeType location_change_type;
-        guint location_change_distance;
-        GFile *pending_location;
-        char *pending_scroll_to;
-        GList *pending_selection;
-        NautilusFile *determine_view_file;
-        GCancellable *mount_cancellable;
-        GError *mount_error;
-        gboolean tried_mount;
+	/* available slots, and active slot.
+ 	 * Both of them may never be NULL.
+ 	 */
+	GList *slots;
+	GList *active_slots;
+	NautilusWindowSlot *active_slot;
 
-        /* View As choices */
-        GtkActionGroup *view_as_action_group; /* owned by ui_manager */
-        GtkRadioAction *view_as_radio_action;
-        GtkRadioAction *extra_viewer_radio_action;
-        guint short_list_merge_id;
-        guint extra_viewer_merge_id;
-        GList *short_list_viewers;
-        char *extra_viewer;
+	NautilusWindowShowHiddenFilesMode show_hidden_files_mode;
 
-        /* Deferred location change. */
-        GFile *location_to_change_to_at_idle;
-        guint location_change_at_idle_id;
+	/* View As menu */
+	GList *short_list_viewers;
+	char *extra_viewer;
 
-        NautilusWindowShowHiddenFilesMode show_hidden_files_mode;
-        gboolean search_mode;
-
-        GCancellable *find_mount_cancellable;
+	/* View As choices */
+	GtkActionGroup *view_as_action_group; /* owned by ui_manager */
+	GtkRadioAction *view_as_radio_action;
+	GtkRadioAction *extra_viewer_radio_action;
+	guint short_list_merge_id;
+	guint extra_viewer_merge_id;
 
 	/* Ensures that we do not react on signals of a
 	 * view that is re-used as new view when its loading
@@ -122,6 +95,9 @@ struct _NautilusNavigationWindowDetails {
         gboolean temporary_search_bar;
 
         GtkWidget *location_button;
+	/* focus widget before the location bar
+	 * has been shown temporarily */
+	GtkWidget *last_focus_widget;
 
         /* Side Pane */
         int side_pane_width;
@@ -132,13 +108,16 @@ struct _NautilusNavigationWindowDetails {
 	guint refresh_go_menu_idle_id;
         guint go_menu_merge_id;
         
+	GtkActionGroup *tabs_menu_action_group;
+	guint tabs_menu_merge_id;
+
         /* Toolbar */
         GtkWidget *toolbar;
         GtkWidget *location_bar;
 
         guint extensions_toolbar_merge_id;
         GtkActionGroup *extensions_toolbar_action_group;
-        
+
 	/* Throbber */
         gboolean    throbber_active;
         GtkWidget  *throbber;
@@ -184,6 +163,7 @@ typedef void (*NautilusBookmarkFailedCallback) (NautilusWindow *window,
                                                 NautilusBookmark *bookmark);
 
 void               nautilus_window_set_status                            (NautilusWindow    *window,
+									  NautilusWindowSlot *slot,
                                                                           const char        *status);
 void               nautilus_window_load_view_as_menus                    (NautilusWindow    *window);
 void               nautilus_window_load_extension_menus                  (NautilusWindow    *window);
@@ -207,23 +187,45 @@ void               nautilus_window_zoom_out                              (Nautil
 void               nautilus_window_zoom_to_level                         (NautilusWindow    *window,
                                                                           NautilusZoomLevel  level);
 void               nautilus_window_zoom_to_default                       (NautilusWindow    *window);
-void		   nautilus_window_show_view_as_dialog			 (NautilusWindow    *window);
-void               nautilus_window_set_content_view_widget               (NautilusWindow    *window,
-                                                                          NautilusView       *content_view);
-void               nautilus_window_set_viewed_file                       (NautilusWindow    *window,
-                                                                          NautilusFile      *file);
+
+NautilusWindowSlot *nautilus_window_open_slot                            (NautilusWindow     *window,
+									  NautilusWindowOpenSlotFlags flags);
+void                nautilus_window_close_slot                           (NautilusWindow     *window,
+									  NautilusWindowSlot *slot);
+
+NautilusWindowSlot *nautilus_window_get_slot_for_view                    (NautilusWindow *window,
+									  NautilusView   *view);
+NautilusWindowSlot *nautilus_window_get_slot_for_content_box             (NautilusWindow *window,
+									  GtkWidget *content_box);
+
+GList *              nautilus_window_get_slots                           (NautilusWindow    *window);
+NautilusWindowSlot * nautilus_window_get_active_slot                     (NautilusWindow    *window);
+void                 nautilus_window_set_active_slot                     (NautilusWindow     *window,
+									  NautilusWindowSlot *slot);
+
 void               nautilus_send_history_list_changed                    (void);
-void               nautilus_window_add_current_location_to_history_list  (NautilusWindow    *window);
 void               nautilus_remove_from_history_list_no_notify           (GFile             *location);
+gboolean           nautilus_add_bookmark_to_history_list                 (NautilusBookmark  *bookmark);
 gboolean           nautilus_add_to_history_list_no_notify                (GFile             *location,
 									  const char        *name,
 									  gboolean           has_custom_name,
 									  GIcon            *icon);
 GList *            nautilus_get_history_list                             (void);
 void               nautilus_window_bookmarks_preference_changed_callback (gpointer           user_data);
-void		   nautilus_window_update_icon				 (NautilusWindow    *window);
 void               nautilus_window_constructed                           (NautilusWindow    *window);
 
+
+/* sync window GUI with current slot. Used when changing slots,
+ * and when updating the slot state.
+ */
+void nautilus_window_sync_status           (NautilusWindow *window);
+void nautilus_window_sync_allow_stop       (NautilusWindow *window,
+					    NautilusWindowSlot *slot);
+void nautilus_window_sync_title            (NautilusWindow *window,
+					    NautilusWindowSlot *slot);
+void nautilus_window_sync_location_widgets (NautilusWindow *window);
+void nautilus_window_sync_zoom_widgets     (NautilusWindow *window);
+
 /* Navigation window menus */
 void               nautilus_navigation_window_initialize_actions                    (NautilusNavigationWindow    *window);
 void               nautilus_navigation_window_initialize_menus                      (NautilusNavigationWindow    *window);
@@ -232,6 +234,9 @@ void               nautilus_navigation_window_remove_bookmarks_menu_callback
 void               nautilus_navigation_window_remove_bookmarks_menu_items           (NautilusNavigationWindow    *window);
 void               nautilus_navigation_window_update_show_hide_menu_items           (NautilusNavigationWindow     *window);
 void               nautilus_navigation_window_update_spatial_menu_item              (NautilusNavigationWindow     *window);
+void               nautilus_navigation_window_update_tab_menu_item_visibility       (NautilusNavigationWindow     *window);
+void               nautilus_navigation_window_sync_tab_menu_title                   (NautilusNavigationWindow     *window,
+										     NautilusWindowSlot           *slot);
 void               nautilus_navigation_window_remove_go_menu_callback    (NautilusNavigationWindow    *window);
 void               nautilus_navigation_window_remove_go_menu_items       (NautilusNavigationWindow    *window);
 
